<?php

use Conto\Quickpay\CallbackData;
use Conto\Quickpay\Enum\QuickpaySettingEnum;
use Conto\Quickpay\JWTUtil;

class QuickpayCallbackModuleFrontController extends ModuleFrontControllerCore
{
    public $ssl = true;

    public function postProcess()
    {
        if ($_SERVER['REQUEST_METHOD'] !== 'POST')
        {
            $this->sendFailResponse("Method not allowed", 405);

            exit();
        }

        try{
            $callbackData = $this->getCallbackData();

            $cartId = (int) $callbackData->getTransactionId();
            $order = OrderCore::getByCartId((int) $cartId);

            if ($order === null) {
                throw new Exception('Transaction Id is incorrect');
            }

            $orderId = $order->id;

            if ($callbackData->isConfirmed()) {

                $stateIdInDb = (int) ConfigurationCore::get('QUICKPAY_CONFIRMED_ID');

                $stateId =
                    OrderStateCore::existsInDatabase($stateIdInDb, 'order_state')
                        ? $stateIdInDb
                        : Configuration::get('PS_OS_PAYMENT');

                $this->changeOrderState($orderId, $stateId);

                $this->sendOkResponse();

                return;
            }

            if ($callbackData->isExpired()) {
                $stateIdInDb = (int) ConfigurationCore::get('QUICKPAY_EXPIRED_ID');

                $stateId =
                    OrderStateCore::existsInDatabase($stateIdInDb, 'order_state')
                        ? $stateIdInDb
                        : Configuration::get('PS_OS_CANCELED');

                $this->changeOrderState($orderId, $stateId);

                $this->sendOkResponse();

                return;
            }

            if ($callbackData->isFailed()) {
                $stateIdInDb = (int) ConfigurationCore::get('QUICKPAY_FAILED_ID');

                $stateId =
                    OrderStateCore::existsInDatabase($stateIdInDb, 'order_state')
                        ? $stateIdInDb
                        : Configuration::get('PS_OS_ERROR');

                $this->changeOrderState($orderId, $stateId);

                $this->sendOkResponse();

                return;
            }

            if ($callbackData->isSigned()) {
                $stateIdInDb = (int) ConfigurationCore::get('QUICKPAY_SIGNED_ID');

                $stateId =
                    OrderStateCore::existsInDatabase($stateIdInDb, 'order_state')
                        ? $stateIdInDb
                        : Configuration::get('PS_OS_PREPARATION');

                $this->changeOrderState($orderId, $stateId);

                $this->sendOkResponse();

                return;
            }
        } catch (Exception $e) {
            $this->sendFailResponse($e->getMessage());
        }
    }

    public function display()
    {
    }

    private function getCallbackData()
    {
        $jwtSecret = Configuration::get('QUICKPAY_SECRET_KEY');
        $jwtSignAlg = JWTUtil::defaultAlg;

        try {
            $jwtToken = $this->parseRequestBody();

            return new CallbackData(JWTUtil::decode($jwtToken, $jwtSecret, $jwtSignAlg));
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function parseRequestBody()
    {
        try {
            $body = file_get_contents('php://input');
            $json = json_decode($body, true);
            $jwtToken = $json['token'];
            restore_error_handler();
            return $jwtToken;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function sendOkResponse()
    {
        header_remove();
        header('Content-Type: application/json');
        echo json_encode(['status' => 'ok']);
    }

    private function sendFailResponse($message, $code=400)
    {
        header_remove();
        header('Content-Type: application/json');
        http_response_code($code);
        echo json_encode(['status' => 'failure', 'message' => $message]);
    }

    private function changeOrderState($orderId, $newOrderStateId)
    {
        $history = new OrderHistoryCore();
        $history->id_order = $orderId;
        $history->id_order_state = $newOrderStateId;
        $history->save();
    }
}
