<?php

use Conto\Quickpay\JWTPayload;
use Conto\Quickpay\JWTUtil;

class QuickpayPaymentModuleFrontController extends ModuleFrontController
{
    public $ssl = true;

    /**
     * @see FrontController::initContent()
     */
    public function initContent()
    {
        parent::initContent();

        if (version_compare(phpversion(), '7.1', '>=')) {
            ini_set( 'serialize_precision', -1 );
        }

        if (!($this->module instanceof Quickpay)) {
            Tools::redirect('index.php?controller=history');

            return;
        }

        $host = JWTPayload::HOST;

        if (!empty(ConfigurationCore::get('QUICKPAY_HOST'))) {
            $host = ConfigurationCore::get('QUICKPAY_HOST');
        }

        $this->cartId = (int) (Tools::getValue('id_cart', 0));
        $this->orderId = (int) OrderCore::getIdByCartId((int) ($this->cartId));
        $cart = CartCore::getCartByOrderId((int) $this->orderId);

        $language = $this->context->language;
        $locale = explode('-', $language->language_code);

        $jwtPayload = JWTPayload::factory()
            ->setAmount($cart->getOrderTotal())
            ->setContractId(ConfigurationCore::get('QUICKPAY_CONTRACT_ID'))
            ->setPaymentPurpose('PS payment ' . $this->cartId)
            ->setTransactionId($this->cartId)
            ->create();

        $widgetToken = JWTUtil::encode($jwtPayload, ConfigurationCore::get('QUICKPAY_SECRET_KEY'), JWTUtil::defaultAlg);

        $this->context->smarty->assign(
            [
                'host' => $host,
                'token' => $widgetToken,
                'redirect_url_success' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'quickpay/redirect?id_order=' . $this->orderId,
                'redirect_url_cancel' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'quickpay/redirect?id_order=' . $this->orderId,
                'lang' => $locale[0],
                'country' => $locale[1],
            ]
        );

        $this->setTemplate("module:quickpay/views/templates/front/payment.tpl");
    }
}
